
#import "Card.h"

/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	Card Globals
|
|	The arrays _CardPipPositions and _CardCardPips provide information about the
|	positioning of pips on cards.  There are 17 possible positions, described in pipPositions,
|	which are selectively refrerred to for each of 13 possible card values by a character string.
|	The letters A-Q are used to represent pips present in the given card.
|
|	The id's _CardFront, _CardBack etc... are images shared by all instances of Card
|
\----------------------------------------------------------------------------------------------------------------------------------*/

static NXPoint		_CardPipPositions [ ] = {
					{ 17, 121 }, { 40, 121 }, { 63, 121 },
					{ 40, 104 },
					{ 40, 95 },
					{ 17, 86 }, { 63, 86 },
					{ 17, 69 }, { 40, 69 }, { 63, 69 },
					{ 17, 52 }, { 63, 52 },
					{ 40, 43 },
					{ 40, 34 },
					{ 17, 17 }, { 40, 17 }, { 63, 17 } } ;

static char		* _CardCardPips [ ] = {
					"I",
					"BP",
					"BIP",
					"ACOQ",
					"ACIOQ",
					"ACHJOQ",
					"ACEHJOQ",
					"ACEHJMOQ",
					"ACFGIKLOQ",
					"ACDFGKLNOQ",
					"CO",
					"CO",
					"AQ" } ;

static id		_CardFrontImage, _CardBackImage, _CardSymbolsImage,
			_CardKingImage, _CardQueenImage, _CardJackImage,
			_CardAceOfSpadesImage ;


@implementation Card

/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	+ initialize
|
|	returns:		(id)		[ super initialize ]
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Initialize the private pasteboard type used by the CardPileView and load all images used
|	by the CardPileView class
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

+ initialize
{
	if ( self == [ Card class ] )
	{
		_CardFrontImage = [ NXImage findImageNamed: "Card" ] ;
		_CardBackImage = [ NXImage findImageNamed: "Back" ] ;
		_CardKingImage = [ NXImage findImageNamed: "King" ] ;
		_CardQueenImage = [ NXImage findImageNamed: "Queen" ] ;
		_CardJackImage = [ NXImage findImageNamed: "Jack" ] ;
		_CardAceOfSpadesImage = [ NXImage findImageNamed: "AceOfSpades" ] ;
		_CardSymbolsImage = [ NXImage findImageNamed: "Symbols" ] ;
	}
	[ super initialize ] ;
	return self ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- init
|
|	returns:		(id)		[ self initSuit:value:upsideDown: ]
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Acts as a cover for initSuit:value:faceUp:
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- init
{
	return [ self initSuit: CS_ACE value: CS_CLUBS faceUp: NO ] ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- initSuit: (CardSuit) aSuit value: (CardValue) aValue
|
|	returns:		(id)		[ self initSuit:value:faceUp: ]
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Acts as a cover for initSuit:value:faceUp:
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- initSuit: (CardSuit) aSuit value: (CardValue) aValue
{
	return [ self initSuit: aSuit value: aValue faceUp: NO ] ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- initSuit: (CardSuit) aSuit value: (CardValue) aValue faceUp: (BOOL) anOrientation
|
|	returns:		(id)		[ super init ]
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Initializes a Card object to the given suit, value and orientation
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- initSuit: (CardSuit) aSuit value: (CardValue) aValue faceUp: (BOOL) anOrientation
{
	suit = aSuit ;
	value = aValue ;
	faceUp = anOrientation ;
	return [ super init ] ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- setSuit: (CardSuit) aSuit
|
|	returns:		(id)		self
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Changes the card suit
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- setSuit: (CardSuit) aSuit
{
	suit = aSuit ;
	return self ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- setValue: (CardValue) aValue
|
|	returns:		(id)		self
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Changes the card value
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- setValue: (CardValue) aValue
{
	value = aValue ;
	return self ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- flip
|
|	returns:		(id)		self
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Turn the card over
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- flip
{
	faceUp = ! faceUp ;
	return self ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- setFaceUp: (BOOL) anOrientation
|
|	returns:		(id)		self
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Turn the card to a specific orientation
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- setFaceUp: (BOOL) anOrientation
{
	faceUp = anOrientation ;
	return self ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- (CardSuit) suit
|
|	returns:		(CardSuit)	The card's suit
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Allow a client to determine the suit of a card
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- (CardSuit) suit
{
	return suit ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- (CardValue) suit
|
|	returns:		(CardValue)	The card's value
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Allow a client to determine the value of a card
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- (CardValue) value
{
	return value ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- (BOOL) isFaceUp
|
|	returns:		(BOOL)		YES if the card is face up
|
|				(BOOL)		NO otherwise
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Allow a client to determine the orientation of a card
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- (BOOL) isFaceUp
{
	return faceUp ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- (CardColor) cardColor
|
|	returns:		(CardColor)	The card's color
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Allow a client to determine the color of a card, as derrived from the suit of the card
|			
\----------------------------------------------------------------------------------------------------------------------------------*/

- (CardColor) cardColor
{
	if ( ( suit == CS_SPADES ) || ( suit == CS_CLUBS ) )
	{
		return CS_BLACK ;
	}
	return CS_RED ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- drawCardAt : (NXPoint *) thePoint
|
|	returns:		(id)		self
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Draw a visual representation of ourself at a given location
|
\----------------------------------------------------------------------------------------------------------------------------------*/

- drawCardAt: (NXPoint *) thePoint
{
	[ self drawOutlineAt: thePoint ] ;
	[ self drawContentsAt: thePoint ] ;
	return self ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- drawOutlineAt : (NXPoint *) thePoint
|
|	returns:		(id)		self
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Draw the outline of a card at a given location
|
\----------------------------------------------------------------------------------------------------------------------------------*/

- drawOutlineAt: (NXPoint *) thePoint
{
	[ _CardFrontImage composite: NX_SOVER toPoint: thePoint ] ;
	return self ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- drawContentsAt : (NXPoint *) thePoint
|
|	returns:		(id)		self
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Draw our contents at a given location
|
\----------------------------------------------------------------------------------------------------------------------------------*/

- drawContentsAt: (NXPoint *) thePoint
{
	if ( ! faceUp )
	{
		
		/*----------------------------------------------------------------------------
		|
		|	Draw the back of the card
		|
		\---------------------------------------------------------------------------*/

		NXPoint		destPoint = { thePoint->x + 8, thePoint->y + 7 } ;

		[ _CardBackImage composite: NX_COPY toPoint: &destPoint ] ;
	}
	else
	{
	
		/*----------------------------------------------------------------------------
		|
		|	Draw the upper left / lower right value indicator
		|
		\---------------------------------------------------------------------------*/

		{
			NXRect		sourceRect = { value * 22, 80, 11, 20 } ;
			NXPoint		destPoint = { thePoint->x + 4, thePoint->y + 141 } ;
			
			if ( ( [ self cardColor ] == CS_RED ) )
			{
				sourceRect.origin.x += 11 ;
			}
			[ _CardSymbolsImage composite: NX_SOVER
					fromRect: &sourceRect toPoint: &destPoint ] ;
			
			sourceRect.origin.y -= 20 ;
			destPoint.x = thePoint->x + 92 ;
			destPoint.y = thePoint->y + 6 ;
			[ _CardSymbolsImage composite: NX_SOVER
					fromRect: &sourceRect toPoint: &destPoint ] ;
		}

		/*----------------------------------------------------------------------------
		|
		|	Draw the upper left / lower right suit indicator
		|
		\---------------------------------------------------------------------------*/

		{
			NXRect		sourceRect = { suit * 13, 15, 13, 15 } ;
			NXPoint		destPoint = { thePoint->x + 3, thePoint->y + 126 } ;
			
			[ _CardSymbolsImage composite: NX_SOVER
					fromRect: &sourceRect toPoint: &destPoint ] ;
			sourceRect.origin.x += 52 ;
			destPoint.x = thePoint->x + 91 ;
			destPoint.y = thePoint->y + 26 ;
			[ _CardSymbolsImage composite: NX_SOVER
					fromRect: &sourceRect toPoint: &destPoint ] ;
		}

		/*----------------------------------------------------------------------------
		|
		|	Draw special card images for face cards and the
		|	ace of spades
		|
		\---------------------------------------------------------------------------*/

		{
			NXPoint		destPoint = { thePoint->x + 16, thePoint->y + 15 } ;

			if ( value == CS_JACK )
			{
				[ _CardJackImage composite: NX_COPY toPoint: &destPoint ] ;
			}
			
			if ( value == CS_QUEEN )
			{
				[ _CardQueenImage composite: NX_COPY toPoint: &destPoint ] ;
			}

			if ( value == CS_KING )
			{
				[ _CardKingImage composite: NX_COPY toPoint: &destPoint ] ;
			}
			
			/*----------------------------------------------------------------------------
			|
			|	Not only draw the special image for the ace of
			|	spades, but avoid the standard suit pip drawing
			|
			\---------------------------------------------------------------------------*/

			if ( ( value == CS_ACE ) && ( suit == CS_SPADES ) )
			{
				[ _CardAceOfSpadesImage composite: NX_COPY toPoint: &destPoint ] ;
				return self ;
			}
		}
		
		/*----------------------------------------------------------------------------
		|
		|	Draw suit pips on the card based on patterns
		|	defined by the _CardCardSymbols and
		|	_CardCardPips arrays
		|
		\---------------------------------------------------------------------------*/

		{
			NXRect		suitRect = { suit * 26, 31, 26, 29 } ;
			NXRect		upsideSuitRect = { 104 + suit * 26, 31, 26, 29 } ;
			char			* pipString = _CardCardPips [ value ] ;

			while ( *pipString )
			{
				NXPoint		tempPoint = _CardPipPositions [ *pipString - 'A' ] ;
				
				tempPoint.x += thePoint->x ;
				tempPoint.y += thePoint->y ;
				[ _CardSymbolsImage composite: NX_SOVER
						fromRect: ( *pipString < 'K' ) ? &suitRect : &upsideSuitRect
						toPoint: &tempPoint ] ;
				pipString++ ;
			}
		}
	}
	return self ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- read: (NXTypedStream *) theStream
|
|	returns:		(int)		self
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Dearchive a Card.
|	
\----------------------------------------------------------------------------------------------------------------------------------*/

- read: (NXTypedStream *) theStream
{
	[ super read: theStream ] ;
	NXReadTypes ( theStream, "iic", &suit, &value, &faceUp ) ;
	return self ;
}


/*-----------------------------------------------------------------------------------------------------------------------------------
|
|	- write: (NXTypedStream *) theStream
|
|	returns:		(int)		self
|
|------------------------------------------------------------------------------------------------------------------------------------
|
|	Archive a Card.
|	
\----------------------------------------------------------------------------------------------------------------------------------*/

- write: (NXTypedStream *) theStream
{
	[ super write: theStream ] ;
	NXWriteTypes ( theStream, "iic", &suit, &value, &faceUp ) ;
	return self ;
}

@end
